///////////////////////////////
#include <acknex.h>
#include <default.c>
#include <d3d9.h>
#include <ackphysx.h>
#include <camera.c>
///////////////////////////////

//moon gravity factors
var earth     =-9.780327;
var moonfactor=16.7;
var moongrav;

var force=4.0;
var thrust=10;
var max_thrust=100;
var min_thrust=0;

var lift;
var throttle=0;
var rotor_radius=250;

VECTOR yaw_force_pos;
VECTOR rolltilt_force_pos;

ENTITY* workercraft;//main craft entity
ENTITY* attached;//attached entity
var has_attached=0;
var total_mass=0;
var shandle;
var score=0;
var height;

SOUND* thrust_snd = "thrust.ogg";
SOUND* ping_snd = "ping.ogg";

MATERIAL* mtl_craft =
{
	ambient_blue = 180;
	ambient_green = 190;
	ambient_red = 190;
	diffuse_blue = 0;
	diffuse_green = 50;
	diffuse_red = 50;
	specular_blue = 150;
	specular_green = 255;
	specular_red = 255;
	power = 10;
}

MATERIAL* mtl_ter =
{
	ambient_blue = 190;
	ambient_green = 190;
	ambient_red = 190;
	diffuse_blue = 150;
	diffuse_green = 150;
	diffuse_red = 150;
	specular_blue = 215;
	specular_green = 255;
	specular_red = 255;
	power = 10;
}

MATERIAL* mtl_thruster =
{
	ambient_blue = 255;
	ambient_green = 0;
	ambient_red = 0;
	diffuse_blue = 0;
	diffuse_green = 0;
	diffuse_red = 0;
	specular_blue = 0;
	specular_green = 0;
	specular_red = 0;
	power = 0;
}

ENTITY* sky =
{
  type = "sky.tga"; 
  layer = 3;	
  u = 10.0;
  v = 80.0;
  flags2 = SKY | DOME | SHOW;	
} 		

ENTITY* sky1 =
{
  type = "sky1.tga"; 
  layer = 2;	
  scale_x = 0.7;	
  u = -10.0;
  v = 80.0;
  flags2 = SKY | DOME | SHOW;	
} 		

ENTITY* sky2 =
{
  type = "sky2.tga"; 
  layer = 1;	
  flags2 = SKY|CUBE|SHOW;	
} 		

PANEL* back=
{
	bmap="physx.bmp";
	layer=1;
	pos_x=400;
	flags=SHOW;
}

function add_force_ent(ENTITY* source,VECTOR* force,VECTOR* pos)
{
	vec_rotate(force,source.pan);
	pXent_addforcelocal(source,force,pos);
}

function blinking_light(ENTITY* source,var vert,VECTOR* col)
{
	VECTOR vpos;
	vec_for_vertex(vpos,source,vert);
	
	ENTITY* sprite=ent_create("light.tga",vpos,NULL);
	set(sprite,PASSABLE);
	set(sprite,LIGHT);	

	vec_scale(sprite.scale_x,0.3);
	vec_set(sprite.blue,col); 

	var time=0;
	while(source!=NULL)
	{
		time+=1*time_step;
		vec_for_vertex(vpos,source,vert);
		vec_set(sprite.x,vpos);

		if(time<10 && time>0)//on
		{
			sprite.lightrange = 150;
			reset(sprite,INVISIBLE);
		}
		if(time>=10)//off
		{
			sprite.lightrange = 0;
			set(sprite,INVISIBLE);
			time=-20;
		}
		wait(1);
	}
}

function attach_event()//attach entity to craft
{
 if (event_type == EVENT_SONAR) 
  {
   	attached=me;
		pXcon_add (PH_HINGE,me,you,0);
		pXcon_setparams1 (me,nullvector,vector(0,0,-1),nullvector);
		pXcon_setparams2 (me,vector(0,0,0), NULL, NULL);
		has_attached=1;
  }
}

function add_ent()//add random junk to carry around
{
	random_seed(0);
	var rnd_ent=integer(random(3));
	ENTITY* ent;
	VECTOR pos;
	vec_set(pos,vector(random(2500)-2500,random(2500)-2500,random(3500)));
	if(rnd_ent==0)	
	ent=ent_create("washer.mdl",pos,NULL);
	if(rnd_ent==1)	
	ent=ent_create("bolt.mdl",pos,NULL);
	if(rnd_ent==2)	
	ent=ent_create("nut.mdl",pos,NULL);
	if(rnd_ent==3)	
	ent=ent_create("bolt.mdl",pos,NULL);

	if(ent)
	{
		var scale=random(2.0);
		if (scale<1.0)scale=1.0;
		vec_scale(ent.scale_x,scale);
		c_setminmax(ent);
		ent.material=mtl_metal;
		set(ent,PASSABLE);
		set(ent,FLAG1);
		ent.emask |= ENABLE_SONAR;
		ent.event=attach_event;
		pXent_settype(ent,PH_RIGID,PH_CONVEX);
		//pXent_setmass(me,random(30));
		pXent_setiterations(ent,1); 
		
		pXent_setfriction(ent,150);
		pXent_setelasticity(ent,0); 
		pXent_setmaxspeed(ent,100);
	}
}

//main craft
action worker_craft()
{
	vec_scale(my.scale_x,0.4);
	c_setminmax(my);
	set(me,PASSABLE);
	set(my,SHADOW);
	my.material=mtl_craft;
	my.ambient=20;

	pXent_settype(my,PH_RIGID,PH_CONVEX);
	pXent_setfriction(my,150);
	pXent_setelasticity(my,0); 
	pXent_setmaxspeed(my,100);
	pXent_setmass(my,1000);
	
	vec_set(yaw_force_pos,vector(-100,0,10));
	vec_set(rolltilt_force_pos,vector(0,0,10));
	
	//add lights
	blinking_light(my,523,vector(0,0,255));
	wait(10);
	blinking_light(my,598,vector(255,0,0));

	ENTITY* upthruster=ent_create("thrust.mdl",nullvector,NULL);
	set(upthruster,TRANSLUCENT);
	set(upthruster,SHADOW|CAST);
	set(upthruster,PASSABLE);
	set(upthruster,FLAG2);
	vec_scale(upthruster.scale_x,0.4);
	upthruster.scale_z=0.5;
	upthruster.material=mtl_thruster;	
	upthruster.alpha=80;
	
	while(1)
	{
		VECTOR vel;
		pXent_getvelocity(my,vel,nullvector);
		vec_scale(vel,0.05);

		height=my.z;
		
		var zerodist=3000-height;//distance to zero gravity

 		draw_text(str_printf(NULL,"forward/backward velocity:%d",(long)vel.x),5,10,COLOR_WHITE);
 		draw_text(str_printf(NULL,"horizontal velocity:%d",(long)vel.y),5,30,COLOR_WHITE);
 		draw_text(str_printf(NULL,"vertical velocity:%d",(long)vel.z),5,50,COLOR_WHITE);
 		draw_text(str_printf(NULL,"distance to zerogravity barrier:%d",(long)zerodist),5,70,COLOR_WHITE);

 		draw_text("up/down: pitch",5,400,COLOR_WHITE);
 		draw_text("left/right: roll",5,420,COLOR_WHITE);
 		draw_text("w/s: thrust up/down",5,440,COLOR_WHITE);
 		draw_text("a/d: thrust yaw",5,460,COLOR_WHITE);
 		draw_text("shift+key: thrust boost",5,480,COLOR_WHITE);
 		draw_text("space: pickup object underneath craft",5,500,COLOR_WHITE);
 		draw_text("enter: shoot attached object",5,520,COLOR_WHITE);
 		draw_text("small simulation of 0 gravity,moon gravity,craft with thrusters\nusing physx and litec\nps.the offset of cog of attached object will effect the crafts cog..total mass is also affected",5,540,COLOR_GREEN);

		if(attached!=NULL)total_mass=pXent_getmass(my)+pXent_getmass(attached);
		else total_mass=pXent_getmass(my);
		draw_text(str_printf(NULL,"total mass:%d",(long)total_mass),5,90,COLOR_WHITE);

		if(zerodist<0)zerodist=0;
		if(height<0)height=0;

		set(upthruster,INVISIBLE);
		
		VECTOR thrustpos;
		vec_set(thrustpos,vector(0,0,-45));
		vec_rotate(thrustpos,my.pan);
		vec_set(upthruster.pan,my.pan);
		vec_add(thrustpos,my.x);
		vec_set(upthruster.x,thrustpos);
		upthruster.scale_z=0.5-random(1.0)*time_step;
		upthruster.alpha=80-random(10.0);
		lift=thrust*throttle/2;
		if(thrust>1)add_force_ent(my,vector(-my.tilt*0.5,-my.roll*0.5,0),rolltilt_force_pos);

		if(key_1) //kill thrusters
		{
//			thrust=0;
		}
		if(key_2)//thrusters on
		{
			thrust=50;
		}

		//pickup objects
		if(key_space)
		{
			if(!has_attached)
			{
				VECTOR aa;
				vec_set(aa,vector(0,0,-100));
				vec_rotate(aa,my.pan);
				vec_add(aa,my.x);
				c_trace(my.x,aa,IGNORE_ME|ACTIVATE_SONAR|IGNORE_CONTENT|IGNORE_FLAG2); 
				draw_line3d(my.x,NULL,100);
				draw_line3d(aa,COLOR_RED,100);
			}
		}
		//drop/shoot objects
		if(key_enter)
		{
			if(has_attached)//shoot attached entity
			{
				pXcon_remove(attached); 
				has_attached=0;
				VECTOR f;
				var m=pXent_getmass(attached);
				vec_set(f,vector(2000*m,-my.roll*0.5,0));
				vec_rotate(f,my.pan);
				pXent_addforcelocal(attached,f,nullvector);
				attached=NULL;
			}
		}
		//up
		if(key_w && thrust>0)
		{
			reset(upthruster,INVISIBLE);
  			
			if(!snd_playing(shandle))
			{
				shandle = ent_playsound(my,thrust_snd,1100);  			
			}
			if(snd_playing(shandle))
			{
				var r=random(15);
				if(r<6)r=6;
				snd_tune(shandle,0,r,0);
			}
			//thrust
			if(thrust>1)add_force_ent(my,vector(-my.tilt*0.5,-my.roll*0.5,250),rolltilt_force_pos);
			//boost
			if(key_shift)add_force_ent(my,vector(-my.tilt*0.5,-my.roll*0.5,260),rolltilt_force_pos);
		}
		//down
		if(key_s && thrust>0)
		{
			if(thrust>1)add_force_ent(my,vector(-my.tilt*0.5,-my.roll*0.5,-250),rolltilt_force_pos);
			//boost
			if(key_shift)add_force_ent(my,vector(-my.tilt*0.5,-my.roll*0.5,-260),rolltilt_force_pos);
		}
		//yaw left
		if(key_a && thrust>1)
		{
			add_force_ent(my,vector(0,-force*2,0),yaw_force_pos);
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,-force*2.1,0),yaw_force_pos);
		}
		//yaw right
		if(key_d && thrust>1)
		{
			add_force_ent(my,vector(0,force*2,0),yaw_force_pos);
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,force*2.1,0),yaw_force_pos);
		}
		//pitch forward
		if(key_cuu && thrust>1)
		{
			add_force_ent(my,vector(0,0,force),vector(-rotor_radius,0,0));
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,0,force+1),vector(-rotor_radius,0,0));
		}
		//pitch backward
		if(key_cud && thrust>1)
		{
			add_force_ent(my,vector(0,0,force),vector(rotor_radius,0,0));
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,0,force+1),vector(rotor_radius,0,0));
		}
		//roll left
		if(key_cul && thrust>1)
		{
			add_force_ent(my,vector(0,0,force),vector(0,-rotor_radius,0));
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,0,force+1),vector(0,-rotor_radius,0));
		}
		//roll right
		if(key_cur && thrust>1)
		{
			add_force_ent(my,vector(0,0,force),vector(0,rotor_radius,0));
			//boost
			if(key_shift)
			add_force_ent(my,vector(0,0,force+1),vector(0,rotor_radius,0));
		}
	
	camera_orbit(my,0,450,50);
	//camera.roll=my.roll;
	wait(1);
	}
}

function magnetron()
{
	my.material=mtl_metal;
	my.ambient=50;
	pXent_settype(my,PH_RIGID,PH_CONVEX);
	
	pXent_setfriction(my,150);
	pXent_setelasticity(my,0); 
	pXent_setmaxspeed(my,100);
	pXent_setmass(my,1000);
}

function main()
{
	vec_set(screen_size,vector(1024,768,0));
	vec_set(screen_color,vector(1,1,1));
	vec_set(sky_color,vector(0,0,0));
	
	video_window(NULL,NULL,0,"");
	d3d_antialias = 1;
	shadow_stencil = 3;
  
	physX_open();

	//setup moon gravity 
	//16.7% of earth's gravity
	moongrav=earth * moonfactor / 100.0;
	pX_setgravity(vector(0,0,moongrav)); 

 	level_load("testmap.wmb");
 	wait(2);

	vec_set(camera.x,vector(-700,0,0));
	camera.arc=90;

	workercraft=ent_create("podhard.mdl",vector(210,0,100),worker_craft);

	ENTITY* ter=ent_for_name("terrain_hmp_000");
	ter.material=mtl_ter;

	ENTITY* e=ent_for_name("boxglass");
	set(e,TRANSLUCENT);
	e.alpha=25;
	e=ent_for_name("boxglass1");
	set(e,TRANSLUCENT);
	e.alpha=25;

	//add some junk
	int i;
	for(i=0;i<100;i++)add_ent();

	ENTITY* magnetron=ent_create("magnetron.mdl",vector(1000,0,3300),magnetron);

	while(1)
	{
		ENTITY* ent;
		draw_text(str_printf(NULL,"score :%d",(long)score),5,110,COLOR_WHITE);

		for(ent = ent_next(NULL);ent;ent=ent_next(ent)) 
		{	
			if(ent!=NULL)
			{
				if(ent.z>3000)//entered zerogravity
				{
					pXent_setbodyflag (ent, NX_BF_DISABLE_GRAVITY,1);
				}
				else//entered moongravity
				{
					pXent_setbodyflag (ent, NX_BF_DISABLE_GRAVITY,0);
				}
				//delete ent when close to magnetron
				if(vec_dist(ent.x,magnetron.x)<=800)
				{
					if(is(ent,FLAG1) && ent!=attached)
					{
						snd_play(ping_snd,100,0);
						score+=pXent_getmass(ent);
						wait(1);
						pXent_settype(ent,0,NULL);
						ent_remove(ent);
					}
				}
			}
		}
	wait(1);
	}
}
